/**
 * Created with WebStorm.
 * User: gp
 * Date: 16-2-26
 *
 */

import expect from 'expect'
import React from 'react'
import testUtils from 'react-addons-test-utils'
import { mount, shallow } from 'enzyme';
import LinkButton from '../../src/page/LinkButton'
import WebDevNavBar from '../../src/page/WebDevNavBar'


describe('LinkButton',()=>{

    describe('component', () => {

        var _onMouseDown = LinkButton.prototype._onMouseDown;
        var _onMouseUp = LinkButton.prototype._onMouseUp;

        function setup(props = {}){

            LinkButton.prototype._onMouseDown = expect.createSpy();
            LinkButton.prototype._onMouseUp = expect.createSpy();

            const renderer = testUtils.createRenderer();
            renderer.render(<LinkButton {...props} />);
            const output = renderer.getRenderOutput();

            return {
                props: props,
                output: output,
                renderer: renderer
            }
        }

        after(() => {
            LinkButton.prototype._onMouseDown = _onMouseDown;
            LinkButton.prototype._onMouseUp = _onMouseUp;
        })

        it('预期初始化渲染按输入内容正常渲染',() => {

            const {output} = setup({
                type: 'button',
                title: 'title',
                color: 'blue',
                icon: 'http://mat1.gtimg.com/www/images/qq2012/dyIcon.png',
                iconStyle: {width:'11px'},
                hide: true
            });

            expect(output.type).toBe('a');
            expect(output.props.style.display).toBe('none');

            const children = output.props.children;
            const [i, span] = children;
            expect(i.type).toBe('i');
            expect(span.type).toBe('span');
            expect(span.props.children).toBe('title');
            expect(span.props.style.color).toBe('blue');

            expect(i.props.style.backgroundImage).toBe('url(http://mat1.gtimg.com/www/images/qq2012/dyIcon.png)');
            expect(i.props.style.width).toBe('11px');

        })


        it('按钮配置disabled为true时， 应该显示disabled配置样式， 事件不可用', () => {
            var props = {
                type: 'button',
                title: [,,'不可用'],
                color: [,,'green'],
                icon: [,,'disabled.png'],
                disabled: true
            }
            const wrapper = shallow(<LinkButton {...props} />);
            expect(wrapper.find('.icon-title').text()).toEqual('不可用');
            expect(wrapper.find('.icon-title').prop('style').color).toEqual('green');
            expect(wrapper.find('.icon').prop('style').backgroundImage).toEqual('url(disabled.png)');

            var instance = wrapper.instance();
            var spy = expect.spyOn(instance,'_onClick');
            var spy1 = expect.spyOn(instance,'_onMouseDown');
            var spy2 = expect.spyOn(instance,'_onMouseUp');
            wrapper.simulate('click');
            expect(spy).toNotHaveBeenCalled();
            expect(spy1).toNotHaveBeenCalled();
            expect(spy2).toNotHaveBeenCalled();
            spy.restore()
            spy1.restore()
            spy2.restore()
            expect.restoreSpies()
        })

        it('点击按钮应该调用 onClick', () => {
            const {output, props} = setup({onClick: expect.createSpy()});
            output.props.onClick();
            expect(props.onClick).toHaveBeenCalled();
        })

        it('鼠标按下时应该调用 _onMouseDown', () => {
            const {output} = setup();
            output.props.onMouseDown();
            expect(LinkButton.prototype._onMouseDown).toHaveBeenCalled();
        })

        it('鼠标抬起时应该调用 _onMouseUp', () => {
            const {output} = setup();
            output.props.onMouseUp();
            expect(LinkButton.prototype._onMouseUp).toHaveBeenCalled();
        })

    })



    describe('私有方法', () => {

        var linkButton = null;

        beforeEach(() => {
            linkButton = new LinkButton({
                type: 'button',
                title: 'title',
                color: 'blue',
                icon: 'http://mat1.gtimg.com/www/images/qq2012/dyIcon.png',
                iconStyle: {width:'11px'},
                hide: true
            });
        })

        it('返回一个初始的states, _buildStates', () => {
            expect(linkButton.state).toEqual({
                title:'title',
                icon: Object.assign(linkButton.theme.icon,{backgroundImage: 'url(http://mat1.gtimg.com/www/images/qq2012/dyIcon.png)'}),
                color: 'blue'
            });
        })

        it('props的iconStyle应该添加到theme的icon上， _processTheme', () => {
            expect(linkButton._processTheme({iconStyle:{width:'10px'}}).icon.width).toBe('10px');
        })

        it('props的hide是true时,display应该是hide， _processTheme', () => {
            expect(linkButton._processTheme({hide:true}).link.display).toBe('none');
        })

        it('props的hide是false时,display应该是flex， _processTheme', () => {
            expect(linkButton._processTheme({hide:false}).link.display).toNotBe('none');
        })

        it('当传入一个字符串时，应该返回一个数组, _str2arr', () => {
            expect(linkButton._str2arr('a')).toEqual(['a']);
            expect(linkButton._str2arr('')).toEqual(['']);
            expect(linkButton._str2arr()).toEqual([]);
            expect(linkButton._str2arr(null)).toEqual([]);
        })

        it('生成的对象的所有属性都应该是数组类型', () => {
             var map = linkButton._warpArray({a:'1', b:null, c:undefined});
             for(key in map){
                 expect(Array.isArray(map[key])).toBe(true);
             }
        })

        it('当color、title、icon都只是一个字符串时，鼠标按下时，状态应该是对应的值, _onMouseDown', () => {

            expect(linkButton._onMouseDown()).toEqual({
               title:'title',
               icon: Object.assign(linkButton.theme.icon,{backgroundImage: 'url(http://mat1.gtimg.com/www/images/qq2012/dyIcon.png)'}),
               color: 'blue'
            });
        })

       it('当color、title、icon都只是一个字符串时，鼠标抬起时，状态应该是对应的值, _onMouseUp', () => {
           expect(linkButton._onMouseUp()).toEqual({
               title:'title',
               icon: Object.assign(linkButton.theme.icon,{backgroundImage: 'url(http://mat1.gtimg.com/www/images/qq2012/dyIcon.png)'}),
               color: 'blue'
           });

        })

       it('当color、title、icon是一个数组时，鼠标按下时，状态应该是对应的第二个值, _onMouseDown', () => {
            linkButton = new LinkButton({
                title: ['title','title1'],
                color: ['blue','blue1'],
                icon: ['./dyIcon.png','./dyIcon1.png']
            });

            expect(linkButton._onMouseDown()).toEqual({
                title:'title1',
                icon: Object.assign(linkButton.theme.icon,{backgroundImage: 'url(./dyIcon1.png)'}),
                color: 'blue1'
            });
        })

       it('当color、title、icon是一个数组时, 鼠标抬起时，状态应该是对应的第一个值, _onMouseUp', () => {
           linkButton = new LinkButton({
               title: ['title','title1'],
               color: ['blue','blue1'],
               icon: ['./dyIcon.png','./dyIcon1.png']
           });

           expect(linkButton._onMouseUp()).toEqual({
               title:'title',
               icon: Object.assign(linkButton.theme.icon,{backgroundImage: 'url(./dyIcon.png)'}),
               color: 'blue'
           });

        })



    })

})

