/**
 * Created with WebStorm.
 * User: gp
 * Date: 16-3-3
 *
 */

import expect from 'expect'
import React from 'react';
import { mount, shallow } from 'enzyme';
import immutable from 'immutable'
import NativeNavBar from '../../src/page/NativeNavBar';
import WeixinNavBar from '../../src/page/WeixinNavBar';
import WebDevNavBar from '../../src/page/WebDevNavBar';

import NavigateBar from '../../src/page/NavigateBar';


describe('NavigateBar', () => {

    var setup = function(env,navOpt={}){
        return shallow(<NavigateBar config={{navOpt:navOpt,scope:{}}} env={env} />);
    }

   it('当前环境是浏览器时， 应该显示 WebDevNavBar', () => {
       const wrapper = setup('browser');
       expect(wrapper.type()).toBe(WebDevNavBar);
   })

    it('当前环境是微信或dingding时， 应该显示 WeixinNavBar', () => {
        let wrapper = setup('weixin');
        expect(wrapper.type()).toBe(WeixinNavBar);

        //dingding 环境检测未提供
        wrapper = setup('dingding');
        expect(wrapper.type()).toBe(WeixinNavBar);
    })

    it('当前环境是native时， 应该显示 NativeNavBar', () => {
        const wrapper = setup('chanjet');
        expect(wrapper.type()).toBe(NativeNavBar);
    })

    it('初始化后右侧组件不能将一个按钮更改成菜单',()=>{
        expect(NavigateBar.prototype.setRightMenu).withContext({navOpt:{right:{type:'button'}}}).withArgs([]).toThrow(/不能将一个按钮更改成菜单/);
    })

    it('初始化后右侧组件不能将一个菜单更改成按钮',()=>{
        expect(NavigateBar.prototype.setRightButton).withContext({navOpt:{right:{type:'menu'}}}).withArgs([]).toThrow(/不能将一个菜单更改成按钮/);
    })

    it('调用setTitle方法时，navbar的标题应该更改成传入的新标题', () => {
        const wrapper = setup().instance();
        wrapper.setTitle('newTitle');
        expect(wrapper.navOpt.title).toBe('newTitle');
    })

    it('调用setRightMenu方法时，navbar的右侧菜单做相应变化', () => {
        const wrapper = setup('',{right:{type:'menu',item:[{text:'tx1',onClick:()=>{}}]}}).instance();
        wrapper.setRightMenu([{text:'txNew',onClick:()=>{}}]);
        expect(wrapper.navOpt.right.item[0].text).toBe('txNew');
    })

    it('调用setRightButton方法时，navbar的右侧按钮做相应变化', () => {
        const wrapper = setup('',{right:{type:'button',item:{text:'tx1',onClick:()=>{}}}}).instance();
        wrapper.setRightButton({text:'txNew',onClick:()=>{}});
        expect(wrapper.navOpt.right.item.text).toBe('txNew');
    })




})
