/**
 * Created with WebStorm.
 * User: gp
 * Date: 16-3-3
 *
 */

import expect from 'expect'
import React from 'react';
import { mount, shallow } from 'enzyme';
import immutable from 'immutable'
import LinkButton from '../../src/page/LinkButton';
import Menu from '../../src/page/Menu';
import WebDevNavBar from '../../src/page/WebDevNavBar'


describe('WebDevNavBar', () => {

    describe('component', () => {

        function setup(props = {}){
            const wrapper = shallow(<WebDevNavBar {...props} />);
            return wrapper;
        }

        describe('render', () => {

            var opts =  {
                    scope:{navigationController:{}},
                    center:{title:'page title'},
                    right:{
                        type: 'menu',
                        item:[
                            {
                                text: 'button',
                                onClick:function () {}
                            },
                            {
                                text: 'button',
                                onClick:function () {}
                            }
                        ]
                    }
            };

            function cloneOpts(){
                return immutable.Map({}).mergeDeep(opts).toJS();
            }

            it('预期按正确的层次结构渲染, 右侧是meun', () => {

                const wrapper = setup(cloneOpts());

                let node = wrapper.node,
                    props = node.props;

                expect(wrapper.type()).toBe('div');
                expect(wrapper.hasClass('navbar')).toBe(true);


                expect(props.children.type).toBe('div');
                expect(props.children.props.className).toBe('navbar-inner');

                let children = wrapper.find('.navbar-inner').children();
                expect(children.length).toBe(3);
                expect(children.first().hasClass('left')).toBe(true);
                expect(children.at(1).hasClass('center')).toBe(true);
                expect(children.last().hasClass('right')).toBe(true)

                expect(wrapper.find('.left').children().first().type()).toBe(LinkButton);
                expect(wrapper.find('.left').children().last().type()).toBe(LinkButton);

                expect(wrapper.find('.right').children().length).toBe(1);
                expect(wrapper.find('.right').children().first().type()).toBe(Menu);
            })

            it('当右侧配置为button配置时,右侧应该是button', () => {
                let opt = cloneOpts();
                opt.right = {
                    type: 'button',
                    item:{text:'btn', onClick:()=>{}}
                }
                const wrapper = setup(opt);
                expect(wrapper.find('.right').children().length).toBe(1);
                expect(wrapper.find('.right').children().first().type()).toBe(LinkButton);
            })

            it('当右侧配置为button配置,并且text和icon都存在时,右侧button应该是text button', () => {
                let opt = cloneOpts();
                opt.right = {
                    type: 'button',
                    item:{text:'btn', icon:'add', onClick:()=>{}}
                }
                const wrapper = setup(opt);
                expect(wrapper.find('.right').children().first().prop('title')).toBe('btn');
                expect(wrapper.find('.right').children().first().prop('icon')).toBe(undefined);
            })

            it('当右侧配置为button配置,并且配置是icon时,右侧button应该是icon button', () => {
                let opt = cloneOpts();
                opt.right = {
                    type: 'button',
                    item:{icon:'add', onClick:()=>{}}
                }
                const wrapper = setup(opt);
                expect(wrapper.find('.right').children().first().prop('icon')).toNotBe('');
                expect(wrapper.find('.right').children().first().prop('text')).toBe(undefined);
            })


            it('返回按钮自定义时间时，点击左侧时，传入的事件onBackClick应该被调用', () => {
                let opt = cloneOpts();
                opt.onBackClick = expect.createSpy();
                const wrapper = setup(opt);

                wrapper.find('.left').children().first().simulate('click');
                expect(opt.onBackClick).toHaveBeenCalled();

            })

            it('左侧未传入onBackClick时，点击左侧时，自身的_goBack应该被调用', () => {
                let opts = cloneOpts();
                const wrapper = setup(opts);
                var spy = expect.spyOn(wrapper.instance(),'_goBack')
                wrapper.find('.left').children().first().simulate('click');
                expect(spy).toHaveBeenCalled();
                spy.restore();
                expect.restoreSpies();
            })

            it('关闭按钮自定义时间时，点击左侧时，传入的事件onCloseClick应该被调用', () => {
                let opt = cloneOpts();
                opt.onCloseClick = expect.createSpy();
                const wrapper = setup(opt);

                wrapper.find('.left').children().last().simulate('click');
                expect(opt.onCloseClick).toHaveBeenCalled();

            })

            it('左侧未传入onCloseClick时，点击左侧时，自身的_goBack应该被调用', () => {
                let opts = cloneOpts();
                const wrapper = setup(opts);
                var spy = expect.spyOn(wrapper.instance(),'_closeWin')
                wrapper.find('.left').children().last().simulate('click');
                expect(spy).toHaveBeenCalled();
                spy.restore();
                expect.restoreSpies();
            })

            it('当前页面是首页时,左侧只有关闭按钮,没有关闭按钮', () => {
                let opt = cloneOpts();
                opt.scope.isRootPage = true;
                const wrapper = setup(opt);
                expect(wrapper.find('.left').children().length).toBe(1);
            })

            it('navBar->right没有配置项时，右侧容器内应该为空', () => {
                let opts = cloneOpts();
                delete opts.right;
                const wrapper = setup(opts);

                expect(wrapper.find('.rights').children().length).toBe(0);
            })

            it('_createRightComponent, navBar->right的components内配置类型不是‘button或menu’时，应该抛出错误', () => {
                expect(WebDevNavBar.prototype._createRightComponent).withArgs({right:{type:'select'}}).toThrow(/只支持"button"/);
            })


        });


    })

})
